@@ -29,6 +29,8 @@
 #include <KColorUtils>
 
 #include <QtGui/QPainter>
+#include <QtGui/QGraphicsSceneMouseEvent>
+#include <QtDBus/QDBusInterface>
 
 SystemLoadViewer::SystemLoadViewer(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args)
@@ -211,6 +213,7 @@ void SystemLoadViewer::constraintsEvent(Plasma::Constraints constraints)
         } else if (backgroundHints() != NoBackground) { // else: Horizontal/Vertical
             setBackgroundHints(NoBackground);
         }
+
     } else if (constraints.testFlag(Plasma::SizeConstraint)) {
         updateSize();
     }
@@ -223,10 +226,12 @@ void SystemLoadViewer::updateSize()
 
     if (formFactor() == Plasma::Horizontal) {
         setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
-        setMinimumWidth(widthForHeight(contentsRect().height()));
+        setMinimumWidth(widthForHeight(size().height()));
     } else if (formFactor() == Plasma::Vertical) {
         setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-        setMinimumHeight(heightForWidth(contentsRect().width()));
+        setMinimumHeight(heightForWidth(size().width()));
+    } else {
+        setPreferredHeight(heightForWidth(size().width()));
     }
 }
 
@@ -274,6 +279,22 @@ QSizeF SystemLoadViewer::sizeHint(Qt::SizeHint which, const QSizeF &constraint)
     return hint;
 }
 
+void SystemLoadViewer::mousePressEvent(QGraphicsSceneMouseEvent *event){
+ 
+    if(event->button() == Qt::LeftButton){
+        m_mousePressLoc = event->screenPos();
+    }
+  
+}
+
+void SystemLoadViewer::mouseReleaseEvent(QGraphicsSceneMouseEvent *event){
+ 
+  if(event->button() == Qt::LeftButton && ((event->screenPos() - m_mousePressLoc).manhattanLength()) < QApplication::startDragDistance()){
+      QDBusInterface("org.kde.krunner", "/App", "org.kde.krunner.App").call(QDBus::NoBlock, "showTaskManager");
+  }
+  
+}
+
 void SystemLoadViewer::createConfigurationInterface(KConfigDialog *parent)
 {
 
@@ -307,8 +328,6 @@ void SystemLoadViewer::createConfigurationInterface(KConfigDialog *parent)
 
     parent->addPage(widGeneral, "General", icon(), QString(), false);
     parent->addPage(widColours, "Colours", icon(), QString(), false);
-
-    parent->setFaceType(KConfigDialog::Tabbed);
 }
 
 void SystemLoadViewer::configUpdated()
@@ -630,18 +649,19 @@ void SystemLoadViewer::toolTipAboutToShow()
 
     // Memory Usage
     const int ramUsage = qRound((m_ramapps / m_ramtotal) * 100);
-    content += i18n("Ram Usage: %1%<br />", ramUsage);
+    int ramTotalMiB = qRound(m_ramtotal / 1024);
+    content += i18n("Ram Usage: %1% of %2 MiB<br />", ramUsage, ramTotalMiB);
 
     // Swap Usage
     if (m_swapAvailable) {
         const int swapUsage = qRound((m_swapused / m_swaptotal) * 100);
-        content += i18n("Swap Usage: %1%", swapUsage);
+        int swapTotalMiB = qRound(m_swaptotal / 10.24); //m_swaptotal is already divided by 100
+        content += i18n("Swap Usage: %1% of %2 MiB", swapUsage, swapTotalMiB);
     } else {
         content += i18nc("not available means the system does not have swap", "Swap: not available");
     }
         
-    Plasma::ToolTipManager::self()->setContent(this,
-        Plasma::ToolTipContent(i18n("System Load Viewer"), content));
+    Plasma::ToolTipManager::self()->setContent(this, Plasma::ToolTipContent(i18n("System Load Viewer"), content, KIcon("view-statistics")));
 }
 
 bool SystemLoadViewer::verticalBars() const
