@@ -21,6 +21,8 @@
 #include "weatherapplet.h"
 
 #include <QLabel>
+#include <QTreeView>
+#include <QHeaderView>
 #include <QApplication>
 #include <QGraphicsLinearLayout>
 #include <QGraphicsGridLayout>
@@ -30,6 +32,7 @@
 #include <KLocale>
 #include <KMessageBox>
 #include <KToolInvocation>
+#include <KUnitConversion/Value>
 
 #include <Plasma/Frame>
 #include <Plasma/IconWidget>
@@ -42,9 +45,9 @@
 #include <weatherview.h>
 #include <cmath>
 
-template <typename T> T clampValue(T value, int decimals) 
-{ 
-        const T mul = std::pow(static_cast<T>(10), decimals); return int(value * mul) / mul; 
+template <typename T> T clampValue(T value, int decimals)
+{
+        const T mul = std::pow(static_cast<T>(10), decimals); return int(value * mul) / mul;
 }
 
 WeatherApplet::WeatherApplet(QObject *parent, const QVariantList &args)
@@ -52,21 +55,20 @@ WeatherApplet::WeatherApplet(QObject *parent, const QVariantList &args)
         m_locationLabel(new Plasma::Label),
         m_forecastTemps(new Plasma::Label),
         m_conditionsLabel(new Plasma::Label),
-        m_currentIcon(new Plasma::IconWidget),
+        m_currentIcon(0),
         m_tempLabel(new Plasma::Label),
         m_windIcon(new Plasma::IconWidget),
         m_courtesyLabel(new Plasma::Label),
+        m_tabBar(new Plasma::TabBar),
         m_fiveDaysModel(0),
         m_detailsModel(0),
-        m_graphicsWidget(0)
+        m_fiveDaysView(0),
+        m_detailsView(0),
+        m_setupLayout(0),
+        m_graphicsWidget(0),
+        m_titleFrame(new Plasma::Frame)
 {
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
-    m_fiveDaysView = 0;
-    m_detailsView = 0;
-    m_tabBar = 0;
-    m_currentIcon = 0;
-    m_titleFrame = 0;
-    m_setupLayout = 0;
     setPopupIcon("weather-not-available");
 }
 
@@ -78,7 +80,6 @@ QGraphicsWidget *WeatherApplet::graphicsWidget()
 void WeatherApplet::init()
 {
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(reloadTheme()));
-
     m_graphicsWidget = new QGraphicsWidget(this);
 
     switch (formFactor()) {
@@ -91,17 +92,15 @@ void WeatherApplet::init()
         break;
     }
 
-    m_titleFrame = new Plasma::Frame(this);
-
     //FIXME: hardcoded quantities, could be better?
     m_titlePanel = new QGraphicsGridLayout;
-    m_titlePanel->setColumnMinimumWidth(0, KIconLoader::SizeHuge);
-    m_titlePanel->setColumnMaximumWidth(0, KIconLoader::SizeHuge * 1.5);
+    //m_titlePanel->setColumnMinimumWidth(0, KIconLoader::SizeHuge);
+    //m_titlePanel->setColumnMaximumWidth(0, KIconLoader::SizeHuge * 1.4);
 
     //these minimum widths seems to give different "weights" when resizing the applet
-    m_titlePanel->setColumnMinimumWidth(1, 10);
-    m_titlePanel->setColumnMinimumWidth(2, 12);
-    m_titlePanel->setColumnMinimumWidth(3, 5);
+    //m_titlePanel->setColumnMinimumWidth(1, 10);
+    //m_titlePanel->setColumnMinimumWidth(2, 12);
+    //m_titlePanel->setColumnMinimumWidth(3, 5);
 
     m_titlePanel->setHorizontalSpacing(0);
     m_titlePanel->setVerticalSpacing(0);
@@ -109,15 +108,15 @@ void WeatherApplet::init()
 
     m_locationLabel->nativeWidget()->setAlignment(Qt::AlignLeft | Qt::AlignAbsolute);
     m_titleFont = QApplication::font();
-    m_titleFont.setPointSize(m_titleFont.pointSize() * 1.6);
+    m_titleFont.setPointSize(m_titleFont.pointSize() * 1.4);
     m_titleFont.setBold(true);
     m_locationLabel->nativeWidget()->setFont(m_titleFont);
     m_locationLabel->nativeWidget()->setWordWrap(false);
-
-    m_locationLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
+    m_locationLabel->setMinimumWidth(85);
 
     m_conditionsLabel->nativeWidget()->setAlignment(Qt::AlignLeft | Qt::AlignAbsolute);
     m_conditionsLabel->nativeWidget()->setWordWrap(false);
+    m_conditionsLabel->setMinimumWidth(55);
 
     m_windIcon->setMaximumSize(0,0);
     m_windIcon->setOrientation(Qt::Horizontal);
@@ -125,28 +124,21 @@ void WeatherApplet::init()
 
     m_tempLabel->nativeWidget()->setAlignment(Qt::AlignRight | Qt::AlignAbsolute);
     m_tempLabel->nativeWidget()->setFont(m_titleFont);
-    m_tempLabel->nativeWidget()->setWordWrap(false);
+
     // This one if a bit crude, ideally we set the horizontal SizePolicy to Preferred, but that doesn't seem
     // to actually respect the minimum size needed to display the temperature. (Bug in Label or QGL?)
-    m_tempLabel->setMinimumWidth(85);
-    m_tempLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    m_tempLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 
     m_forecastTemps->nativeWidget()->setAlignment(Qt::AlignRight | Qt::AlignAbsolute);
     m_forecastTemps->nativeWidget()->setWordWrap(false);
     m_forecastTemps->nativeWidget()->setFont(KGlobalSettings::smallestReadableFont());
     m_forecastTemps->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 
-    /*QGraphicsWidget *titleSpacer = new QGraphicsWidget(this);
-    //FIXME: will be a width somewhat related to the weather icon size
-    titleSpacer->setPreferredWidth(60);
-    titleSpacer->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-    m_titlePanel->addItem(titleSpacer, 0, 0, 2, 1);*/
-
-    m_titlePanel->addItem(m_locationLabel, 0, 0, 1, 3);
-    m_titlePanel->addItem(m_tempLabel, 0, 3);
-    m_titlePanel->addItem(m_conditionsLabel, 1, 0);
-    m_titlePanel->addItem(m_windIcon, 1, 2);
-    m_titlePanel->addItem(m_forecastTemps, 1, 3);
+    m_titlePanel->setColumnMinimumWidth(0, KIconLoader::SizeHuge);
+    m_titlePanel->addItem(m_locationLabel, 0, 1);
+    m_titlePanel->addItem(m_tempLabel, 0, 4);
+    m_titlePanel->addItem(m_conditionsLabel, 1, 1);
+    m_titlePanel->addItem(m_forecastTemps, 1, 4);
 
     m_titlePanel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
@@ -154,12 +146,12 @@ void WeatherApplet::init()
     m_layout->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
     m_layout->setPreferredSize(400,300);
 
-    // This seems broken with layouts :(
     m_titleFrame->setLayout(m_titlePanel);
     m_layout->addItem(m_titleFrame);
 
     m_courtesyLabel->nativeWidget()->setWordWrap(false);
     m_courtesyLabel->nativeWidget()->setAlignment(Qt::AlignRight);
+    m_courtesyLabel->nativeWidget()->setFont(KGlobalSettings::smallestReadableFont());
     connect(m_courtesyLabel, SIGNAL(linkActivated(QString)), this, SLOT(invokeBrowser(QString)));
 
     m_graphicsWidget->setLayout(m_layout);
@@ -186,6 +178,58 @@ void WeatherApplet::toolTipAboutToShow()
     Plasma::ToolTipManager::self()->setContent(this, data);
 }
 
+void WeatherApplet::resizeView()
+{
+    if (m_fiveDaysView) {
+        int totalColumns = m_fiveDaysView->nativeWidget()->header()->count();
+        QString maxString, curString;
+        int longestString = 0;
+
+        kDebug() << "Total Columns: " << totalColumns;
+        // Figure out maximum text length in model this will give us width sizing we want with setWidthSize
+        for (int i = 0; i < totalColumns; i++) {
+             curString  = m_fiveDaysView->model()->index(0, i).data(Qt::DisplayRole).toString();
+             if (m_fiveDaysView->nativeWidget()->fontMetrics().width(curString) > m_fiveDaysView->nativeWidget()->fontMetrics().width(maxString)) {
+                 maxString = curString;
+                 longestString = m_fiveDaysView->nativeWidget()->fontMetrics().width("XX"+maxString);
+             }
+        }
+        kDebug() << "Maximum string is: " << maxString;
+        kDebug() << "Longest Size: " << longestString;
+        kDebug() << "m_fiveDaysView width is: " << m_fiveDaysView->size().width();
+
+        int maxColumns = m_fiveDaysView->size().width() / longestString;
+        int shownColumns = 0;
+        for (int i = 0; i < totalColumns; i++) {
+             if (!m_fiveDaysView->nativeWidget()->isColumnHidden(i)) {
+                 shownColumns++;
+                 kDebug() << "Column " << i << " is NOT hidden";
+             }
+        }
+        int difference = 0;
+        kDebug() << "Maximum Column Width: " << maxColumns;
+        kDebug() << "shown Columns: " << shownColumns;
+        if (maxColumns < shownColumns) {
+            difference = qAbs(maxColumns-shownColumns);
+            kDebug() << "A: Difference is:" << difference;
+            for (int i = maxColumns; i < shownColumns; ++i) {
+                 kDebug() << "HIDE: i = " << i;
+                 m_fiveDaysView->nativeWidget()->setColumnHidden(i, true);
+            }
+        } else {
+            difference = qAbs(shownColumns-maxColumns);
+            if (difference > totalColumns) {
+                kDebug() << "Difference is bigger than totalColumns!";
+            }
+            kDebug() << "B: Difference is: " << difference;
+            for (int i = difference; i < maxColumns; ++i) {
+                 kDebug() << "UNHIDE: i = " << i;
+                 m_fiveDaysView->nativeWidget()->setColumnHidden(i, false);
+            }
+        }
+    }
+}
+
 void WeatherApplet::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::FormFactorConstraint) {
@@ -198,10 +242,13 @@ void WeatherApplet::constraintsEvent(Plasma::Constraints constraints)
             Plasma::ToolTipManager::self()->unregisterWidget(this);
             break;
         }
+    } else if (constraints & Plasma::SizeConstraint) {
+        resizeView();
+        update();
     }
 }
 
-void WeatherApplet::invokeBrowser(const QString& url)
+void WeatherApplet::invokeBrowser(const QString& url) const
 {
     KToolInvocation::invokeBrowser(url);
 }
@@ -216,7 +263,7 @@ void WeatherApplet::setVisible(bool visible, QGraphicsLayout *layout)
     }
 }
 
-void WeatherApplet::setVisibleLayout(bool val)  
+void WeatherApplet::setVisibleLayout(bool val)
 {
     if (m_titleFrame) {
         m_titleFrame->setVisible(val);
@@ -228,28 +275,30 @@ void WeatherApplet::setVisibleLayout(bool val)
     m_courtesyLabel->setVisible(val);
 }
 
-QString WeatherApplet::convertTemperature(int format, QString value, int type, bool rounded = false, bool degreesOnly = false)
+QString WeatherApplet::convertTemperature(KUnitConversion::UnitPtr format, QString value,
+                                          int type, bool rounded = false, bool degreesOnly = false)
 {
-    double temp = WeatherUtils::convertTemperature(value.toDouble(), type, format);
+    KUnitConversion::Value v(value.toDouble(), type);
+    v = v.convertTo(format);
 
     if (rounded) {
-        int tempNumber = qRound(temp);
+        int tempNumber = qRound(v.number());
         if (degreesOnly) {
-            return i18nc("temperature, unit", "%1%2", tempNumber, WeatherUtils::getUnitString(WeatherUtils::DegreeUnit, false));
+            return i18nc("temperature, unit", "%1%2", tempNumber, i18nc("Degree, unit symbol", "°"));
         } else {
-            return i18nc("temperature, unit", "%1%2", tempNumber, WeatherUtils::getUnitString(format, false));
+            return i18nc("temperature, unit", "%1%2", tempNumber, v.unit()->symbol());
         }
     } else {
-        float formattedTemp = clampValue(temp, 1);
+        float formattedTemp = clampValue(v.number(), 1);
         if (degreesOnly) {
-            return i18nc("temperature, unit", "%1%2", formattedTemp, WeatherUtils::getUnitString(WeatherUtils::DegreeUnit, false));
+            return i18nc("temperature, unit", "%1%2", formattedTemp, i18nc("Degree, unit symbol", "°"));
         } else {
-            return i18nc("temperature, unit", "%1%2", formattedTemp, WeatherUtils::getUnitString(format, false));
+            return i18nc("temperature, unit", "%1%2", formattedTemp, v.unit()->symbol());
         }
     }
 }
 
-bool WeatherApplet::isValidData(const QVariant &data)
+bool WeatherApplet::isValidData(const QVariant &data) const
 {
     return ((data != "N/A") && (!data.toString().isEmpty()));
 }
@@ -263,72 +312,86 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
         // fiveDayTokens[3] = High Temperature
         // fiveDayTokens[4] = Low Temperature
         if (fiveDayTokens[4] != "N/A" && fiveDayTokens[3] == "N/A") {  // Low temperature
-            m_tempLabel->setText(convertTemperature(temperatureUnitInt(), data["Temperature"].toString(), data["Temperature Unit"].toInt(), false));
-            m_forecastTemps->setText(i18nc("Low temperature", "Low: %1", convertTemperature(temperatureUnitInt(), fiveDayTokens[4], data["Temperature Unit"].toInt())));
+            m_tempLabel->setText(convertTemperature(temperatureUnit(), data["Temperature"].toString(), data["Temperature Unit"].toInt(), false));
+            m_forecastTemps->setText(i18nc("Low temperature", "Low: %1", convertTemperature(temperatureUnit(), fiveDayTokens[4], data["Temperature Unit"].toInt(), true)));
         } else if (fiveDayTokens[3] != "N/A" && fiveDayTokens[4] == "N/A") { // High temperature
-            m_forecastTemps->setText(i18nc("High temperature", "High: %1", convertTemperature(temperatureUnitInt(), fiveDayTokens[3], data["Temperature Unit"].toInt())));
+            m_forecastTemps->setText(i18nc("High temperature", "High: %1", convertTemperature(temperatureUnit(), fiveDayTokens[3], data["Temperature Unit"].toInt(), true)));
         } else { // Both high and low
-            m_forecastTemps->setText(i18nc("High & Low temperature", "H: %1 L: %2", convertTemperature(temperatureUnitInt(), fiveDayTokens[3], data["Temperature Unit"].toInt()), convertTemperature(temperatureUnitInt(),  fiveDayTokens[4], data["Temperature Unit"].toInt())));
+            m_forecastTemps->setText(i18nc("High & Low temperature", "H: %1 L: %2", convertTemperature(temperatureUnit(), fiveDayTokens[3], data["Temperature Unit"].toInt(), true), convertTemperature(temperatureUnit(),  fiveDayTokens[4], data["Temperature Unit"].toInt(), true)));
         }
     }
     else {
         m_forecastTemps->setText(QString());
     }
 
-    m_conditionsLabel->setText(data["Current Conditions"].toString());
+    m_conditionsLabel->setText(data["Current Conditions"].toString().trimmed());
 
     if (isValidData(data["Temperature"])) {
-        m_tempLabel->setText(convertTemperature(temperatureUnitInt(), data["Temperature"].toString(), data["Temperature Unit"].toInt(), false));
-
+        m_tempLabel->setText(convertTemperature(temperatureUnit(), data["Temperature"].toString(), data["Temperature Unit"].toInt(), false));
     } else {
-        m_tempLabel->setText(QString());
+        m_tempLabel->setText(i18nc("Not available","N/A"));
     }
 
     if (!m_currentIcon) {
-        kDebug() << "Create new Plasma::IconWidget";
-        m_currentIcon = new Plasma::IconWidget(this);
-        m_currentIcon->setMaximumWidth(KIconLoader::SizeEnormous);
-        m_currentIcon->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+        kDebug() << "Create new Plasma::IconWidget (condition)";
+        m_currentIcon = new Plasma::IconWidget(m_graphicsWidget);
+        m_currentIcon->resize(KIconLoader::SizeEnormous, KIconLoader::SizeEnormous);
+        m_currentIcon->setMinimumWidth(KIconLoader::SizeSmall);
+        m_currentIcon->setZValue(900);
+        m_currentIcon->setPos(0,0);
         //m_currentIcon = new Plasma::IconWidget(KIcon(data["Condition Icon"].toString()), QString(), this);
         //m_currentIcon->icon().pixmap(QSize(KIconLoader::SizeEnormous,KIconLoader::SizeEnormous));
         m_currentIcon->setDrawBackground(false);
+        m_currentIcon->setAcceptHoverEvents(false);
+        m_currentIcon->show();
     }
+
+    if (!m_windIcon) {
+        kDebug() << "Create new Plasma::IconWidget (wind)";
+        m_windIcon = new Plasma::IconWidget();
+        m_windIcon->setOrientation(Qt::Horizontal);
+        m_windIcon->setDrawBackground(false);
+    }
+
+    if (data["Wind Speed"] != "N/A" && data["Wind Speed"].toDouble() != 0 && data["Wind Speed"] != "Calm") {
+        KUnitConversion::Value v(data["Wind Speed"].toDouble(), data["Wind Speed Unit"].toInt());
+        v = v.convertTo(speedUnit());
+        m_windIcon->setText(i18nc("wind direction, speed","%1 %2 %3", data["Wind Direction"].toString(),
+                clampValue(v.number(), 1), v.unit()->symbol()));
+    } else {
+        if (data["Wind Speed"] == "N/A") {
+            m_windIcon->setText(i18nc("Not available","N/A"));
+        } else {
+            if (data["Wind Speed"].toInt() == 0 || data["Wind Speed"] == "Calm") {
+                m_windIcon->setText(i18nc("Wind condition","Calm"));
+            }
+        }
+    }
+
     m_courtesyLabel->setText(data["Credit"].toString());
 
     if (!data["Credit Url"].toString().isEmpty()) {
         QString creditUrl = QString("<A HREF=\"%1\" ><FONT size=\"-0.5\" color=\"%2\">%3</FONT></A>").arg(data["Credit Url"].toString()).arg(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor).rgb()).arg(data["Credit"].toString());
         m_courtesyLabel->nativeWidget()->setTextInteractionFlags(Qt::TextBrowserInteraction);
-        //m_courtesyLabel->setAcceptHoverEvents(true);
-        //m_courtesyLabel->nativeWidget()->setMouseTracking(true);
-
         m_courtesyLabel->setText(creditUrl);
     }
 
     if (data["Condition Icon"].toString() == "N/A") {
         m_currentIcon->setIcon(KIcon("weather-not-available"));
         setPopupIcon("weather-not-available");
+    } else if (!isValidData(data["Condition Icon"]) || data["Condition Icon"].toString() == "N/U") {
+        m_currentIcon->hide();
     } else {
         m_currentIcon->setIcon(KIcon(data["Condition Icon"].toString()));
+        m_currentIcon->show();
         setPopupIcon(data["Condition Icon"].toString());
     }
 
-    if (!m_tabBar) {
-        m_tabBar = new Plasma::TabBar(this);
-        m_tabBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-        kDebug() << "Create new Plasma::TabBar";
-    }
+    m_tabBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
-    if (m_tabBar->count() > 0) {
-        // If we have items in tab clean it up first
-        while (m_tabBar->count()) {
-             m_tabBar->removeTab(0);
-        }
-    }
-
-    // FIXME: Destroy the treeview because if we don't Plasma crashes?:w
-    if (m_fiveDaysView) {
-        delete m_fiveDaysView;
-        m_fiveDaysView = 0;
+    // If we have items in tab clean it up first
+    while (m_tabBar->count()) {
+        m_tabBar->takeTab(0);
     }
 
     // If we have a 5 day forecast, display it
@@ -336,8 +399,7 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
         if (!m_fiveDaysView) {
             kDebug() << "Create 5 Days Plasma::WeatherView";
             m_fiveDaysView = new Plasma::WeatherView(m_tabBar);
-            m_fiveDaysView->setHasHeader(true);
-            m_fiveDaysView->setOrientation(Qt::Vertical);
+            connect(m_fiveDaysView->nativeWidget()->header(), SIGNAL(sectionResized(int, int, int)), this, SLOT(fiveDaysColumnResized(int, int, int)));
         }
 
         if (!m_fiveDaysModel) {
@@ -378,7 +440,7 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
 
             titleFont.setBold(true);
             dayName->setFont(titleFont);
-            dayName->setText(fiveDayTokens[0]);
+            dayName->setText(fiveDayTokens[0].trimmed());
             dayItems.append(dayName);
 
             // If we see N/U (Not Used) we skip the item
@@ -410,7 +472,7 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
                     highItem->setText(i18nc("Short for no data available","-"));
                     hiItems.append(highItem);
                 } else {
-                    highItem->setText(convertTemperature(temperatureUnitInt(), fiveDayTokens[3], data["Temperature Unit"].toInt()));
+                    highItem->setText(convertTemperature(temperatureUnit(), fiveDayTokens[3], data["Temperature Unit"].toInt(), true));
                     hiItems.append(highItem);
                 }
             }
@@ -423,14 +485,12 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
                     lowItem->setText(i18nc("Short for no data available","-"));
                     lowItems.append(lowItem);
                 } else {
-                    lowItem->setText(convertTemperature(temperatureUnitInt(), fiveDayTokens[4], data["Temperature Unit"].toInt()));
+                    lowItem->setText(convertTemperature(temperatureUnit(), fiveDayTokens[4], data["Temperature Unit"].toInt(), true));
                     lowItems.append(lowItem);
                 }
             }
-
         }
 
-
         if (dayItems.count() > 0) {
             m_fiveDaysModel->appendRow(dayItems);
         }
@@ -451,15 +511,19 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
             // If we have any items, display 5 Day tab, otherwise only details
             QString totalDays = i18ncp("Forecast period timeframe", "1 Day", "%1 Days", data["Total Weather Days"].toInt());
             m_tabBar->addTab(totalDays, m_fiveDaysView);
+        } else {
+            delete m_fiveDaysView;
+            m_fiveDaysView = 0;
         }
+    } else {
+        delete m_fiveDaysView;
+        m_fiveDaysView = 0;
     }
 
     // Details data
     if (!m_detailsView) {
         kDebug() << "Create Details Plasma::WeatherView";
         m_detailsView = new Plasma::WeatherView(m_tabBar);
-        m_detailsView->setHasHeader(false);
-        m_detailsView->setOrientation(Qt::Horizontal);
     }
 
     if (!m_detailsModel) {
@@ -471,9 +535,9 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
 
     if (isValidData(data["Windchill"])) {
        QStandardItem *dataWindchill = new QStandardItem();
-       
+
        // Use temperature unit to convert windchill temperature we only show degrees symbol not actual temperature unit
-       dataWindchill->setText(i18nc("windchill, unit", "Windchill: %1", convertTemperature(temperatureUnitInt(), data["Windchill"].toString(), data["Temperature Unit"].toInt(), false, true)));
+       dataWindchill->setText(i18nc("windchill, unit", "Windchill: %1", convertTemperature(temperatureUnit(), data["Windchill"].toString(), data["Temperature Unit"].toInt(), false, true)));
        m_detailsModel->appendRow(dataWindchill);
     }
 
@@ -481,20 +545,22 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
         QStandardItem *dataHumidex = new QStandardItem();
 
         // Use temperature unit to convert humidex temperature we only show degrees symbol not actual temperature unit
-        dataHumidex->setText(i18nc("humidex, unit","Humidex: %1", convertTemperature(temperatureUnitInt(), data["Humidex"].toString(), data["Temperature Unit"].toInt(), false, true)));
+        dataHumidex->setText(i18nc("humidex, unit","Humidex: %1", convertTemperature(temperatureUnit(), data["Humidex"].toString(), data["Temperature Unit"].toInt(), false, true)));
 
         m_detailsModel->appendRow(dataHumidex);
     }
 
     if (isValidData(data["Dewpoint"])) {
         QStandardItem *dataDewpoint = new QStandardItem();
-        dataDewpoint->setText(i18nc("ground temperature, unit", "Dewpoint: %1", convertTemperature(temperatureUnitInt(), data["Dewpoint"].toString(), data["Dewpoint Unit"].toInt(), false)));
+        dataDewpoint->setText(i18nc("ground temperature, unit", "Dewpoint: %1", convertTemperature(temperatureUnit(), data["Dewpoint"].toString(), data["Temperature Unit"].toInt(), false)));
         m_detailsModel->appendRow(dataDewpoint);
     }
 
     if (isValidData(data["Pressure"])) {
         QStandardItem *dataPressure = new QStandardItem();
-        dataPressure->setText(i18nc("pressure, unit","Pressure: %1 %2", clampValue(WeatherUtils::convertPressure(data["Pressure"].toDouble(), data["Pressure Unit"].toInt(), pressureUnitInt()), 2), pressureUnit()));
+        KUnitConversion::Value v(data["Pressure"].toDouble(), data["Pressure Unit"].toInt());
+        v = v.convertTo(pressureUnit());
+        dataPressure->setText(i18nc("pressure, unit","Pressure: %1 %2", clampValue(v.number(), 2), v.unit()->symbol()));
         m_detailsModel->appendRow(dataPressure);
     }
 
@@ -507,10 +573,11 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
     if (isValidData(data["Visibility"])) {
         QStandardItem *dataVisibility = new QStandardItem();
         bool isNumeric;
-        double visibility = data["Visibility"].toDouble(&isNumeric);
-        Q_UNUSED(visibility)
+        data["Visibility"].toDouble(&isNumeric);
         if (isNumeric) {
-            dataVisibility->setText(i18nc("distance, unit","Visibility: %1 %2", clampValue(WeatherUtils::convertDistance(data["Visibility"].toDouble(), data["Visibility Unit"].toInt(), visibilityUnitInt()), 1), visibilityUnit()));
+            KUnitConversion::Value v(data["Visibility"].toDouble(), data["Visibility Unit"].toInt());
+            v = v.convertTo(visibilityUnit());
+            dataVisibility->setText(i18nc("distance, unit","Visibility: %1 %2", clampValue(v.number(), 1), v.unit()->symbol()));
         } else {
             dataVisibility->setText(i18nc("visibility from distance", "Visibility: %1", data["Visibility"].toString()));
         }
@@ -520,31 +587,30 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
 
     if (isValidData(data["Humidity"])) {
         QStandardItem *dataHumidity = new QStandardItem();
-        dataHumidity->setText(i18nc("content of water in air", "Humidity: %1", data["Humidity"].toString()));
+        dataHumidity->setText(i18nc("content of water in air", "Humidity: %1%2", data["Humidity"].toString(), i18nc("Precent, measure unit", "%")));
         m_detailsModel->appendRow(dataHumidity);
     }
 
-    if (data["Wind Speed"] != "N/A" && data["Wind Speed"].toDouble() != 0 && data["Wind Speed"] != "Calm") {
-        m_windIcon->setText(i18nc("wind direction, speed","%1 %2 %3", data["Wind Direction"].toString(),
-                clampValue(WeatherUtils::convertSpeed(data["Wind Speed"].toDouble(), data["Wind Speed Unit"].toInt(), speedUnitInt()), 1), speedUnit()));
-    } else {
-        if (data["Wind Speed"] == "N/A") {
-            m_windIcon->setText(i18nc("Not available","N/A"));
-        } else {
-            if (data["Wind Speed"].toInt() == 0 || data["Wind Speed"] == "Calm") {
-                m_windIcon->setText(i18nc("Wind condition","Calm"));
-            }
-        }
-    }
+    if (isValidData(data["Wind Speed"])) {
+        Plasma::Svg svgWindIcon;
+        svgWindIcon.setImagePath("weather/wind-arrows");
+        QIcon windIcon = svgWindIcon.pixmap(data["Wind Direction"].toString());
+        m_windIcon->setIcon(windIcon);
+        m_windIcon->setMaximumSize(m_windIcon->sizeFromIconSize(KIconLoader::SizeSmall));
+        m_windIcon->update();
 
-    m_windIcon->setSvg("weather/wind-arrows", data["Wind Direction"].toString());
-    m_windIcon->setMaximumSize(m_windIcon->sizeFromIconSize(KIconLoader::SizeSmall));
-    m_windIcon->update();
+        QStandardItem *windInfo = new QStandardItem(m_windIcon->icon(), NULL);
+        windInfo->setTextAlignment(Qt::AlignRight);
+        windInfo->setText(m_windIcon->text());
+        m_detailsModel->appendRow(windInfo);
+    }
 
     if (isValidData(data["Wind Gust"])) {
         // Convert the wind format for nonstandard types
         QStandardItem *dataGust = new QStandardItem();
-        dataGust->setText(i18nc("winds exceeding wind speed briefly", "Wind Gust: %1 %2", clampValue(WeatherUtils::convertSpeed(data["Wind Gust"].toDouble(), data["Wind Gust Unit"].toInt(), speedUnitInt()), 1), speedUnit()));
+        KUnitConversion::Value v(data["Wind Gust"].toDouble(), data["Wind Gust Unit"].toInt());
+        v = v.convertTo(speedUnit());
+        dataGust->setText(i18nc("winds exceeding wind speed briefly", "Wind Gust: %1 %2", clampValue(v.number(), 1), v.unit()->symbol()));
         m_detailsModel->appendRow(dataGust);
     }
 
@@ -591,6 +657,7 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
                 warningLayout->addItem(warnNotice, rowCount, 0);
                 rowCount++;
             }
+
             noticeLayout->addItem(warningLayout);
         }
 
@@ -634,16 +701,21 @@ void WeatherApplet::weatherContent(const Plasma::DataEngine::Data &data)
     }
 
     if (!m_setupLayout) {
-        m_bottomLayout->addItem(m_currentIcon);
+        //m_bottomLayout->addItem(m_currentIcon);
+        QGraphicsWidget *spacer = new QGraphicsWidget(this);
+        spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+        spacer->setMinimumWidth(KIconLoader::SizeMedium);
+        spacer->setMaximumWidth(KIconLoader::SizeHuge);
+        m_bottomLayout->addItem(spacer);
         m_bottomLayout->addItem(m_tabBar);
+        m_bottomLayout->setStretchFactor(m_tabBar, 2);
         m_layout->addItem(m_bottomLayout);
         m_layout->addItem(m_courtesyLabel);
         m_setupLayout = 1;
     }
-    update();
 
     setVisibleLayout(true);
-    //updateGeometry();
+    update();
 }
 
 void WeatherApplet::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
@@ -655,15 +727,15 @@ void WeatherApplet::dataUpdated(const QString &source, const Plasma::DataEngine:
     m_currentData = data;
     setVisibleLayout(false);
     weatherContent(data);
-    update();
     WeatherPopupApplet::dataUpdated(source, data);
+    update();
 }
 
 void WeatherApplet::configAccepted()
 {
     setVisibleLayout(false);
     WeatherPopupApplet::configAccepted();
-}   
+}
 
 void WeatherApplet::reloadTheme()
 {
